@@ -13,8 +13,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""OTT ground geometries."""
+"""OTT ground geometries: Classes and cost functions to instantiate them."""
+from . import costs
 from . import epsilon_scheduler
 from . import geometry
 from . import grid
+from . import ops
 from . import pointcloud
