@@ -97,7 +97,6 @@ class Euclidean(CostFn):
 @jax.tree_util.register_pytree_node_class
 class Bures(CostFn):
   """Bures distance between a pair of (mean, cov matrix) raveled as vectors."""
-  # TODO(cuturi) add regularized / unbalanced https://arxiv.org/abs/2006.02572
 
   def __init__(self, dimension, **kwargs):
     super().__init__()
@@ -134,3 +133,85 @@ class Bures(CostFn):
   def tree_unflatten(cls, aux_data, children):
     del children
     return cls(aux_data[0], **aux_data[1])
+
+
+@jax.tree_util.register_pytree_node_class
+class UnbalancedBures(CostFn):
+  """Regularized, unbalanced Bures distance between triplets.
+
+  This cost implements the value defined in https://arxiv.org/pdf/2006.02572.pdf
+  Equation 37, 39, 40. We follow their notations. It is assumed inputs are given
+  as triplets (mass, mean, covariance) raveled as vectors, in that order.
+  """
+
+  def __init__(self,
+               dimension: int,
+               gamma: float = 1,
+               sigma: float = 1,
+               **kwargs):
+    super().__init__()
+    self._dimension = dimension
+    self._gamma = gamma
+    self._sigma2 = sigma ** 2
+    self._sqrtm_kw = kwargs
+
+  def norm(self, x):
+    return self._gamma * x[0]
+
+  def pairwise(self, x, y):
+    # Sets a few constants
+    gam = self._gamma
+    sig2 = self._sigma2
+    lam = sig2 + gam / 2
+    tau = gam / (2 * lam)
+
+    # Extracts mass, mean vector, covariance matrices
+    mass_x, mass_y = x[0], y[0]
+    diff_means = x[1:1+self._dimension] - y[1:1+self._dimension]
+    x_mat = jnp.reshape(x[1+self._dimension:],
+                        (self._dimension, self._dimension))
+    y_mat = jnp.reshape(y[1+self._dimension:],
+                        (self._dimension, self._dimension))
+
+    # Identity matrix of suitable size
+    iden = jnp.eye(self._dimension)
+
+    # Creates matrices needed in the computation
+    tilde_a = 0.5 * gam * (iden - lam * jnp.linalg.inv(x_mat + lam * iden))
+    tilde_b = 0.5 * gam * (iden - lam * jnp.linalg.inv(y_mat + lam * iden))
+
+    tilde_a_b = jnp.matmul(tilde_a, tilde_b)
+    c_mat = matrix_square_root.sqrtm(
+        1 / tau * tilde_a_b + 0.25 * (sig2 ** 2) * iden, **self._sqrtm_kw)[0]
+    c_mat -= 0.5 * sig2 * iden
+
+    # Computes log determinants (their sign should be >0).
+    sldet_c, ldet_c = jnp.linalg.slogdet(c_mat)
+    sldet_t_ab, ldet_t_ab = jnp.linalg.slogdet(tilde_a_b)
+    sldet_ab, ldet_ab = jnp.linalg.slogdet(jnp.matmul(x_mat, y_mat))
+    sldet_c_ab, ldet_c_ab = jnp.linalg.slogdet(c_mat - 2 * tilde_a_b / gam)
+
+    # Gathers all these results to compute log total mass of transport
+    log_m_pi = (0.5 * self._dimension * sig2 / (gam + sig2)) * jnp.log(sig2)
+
+    log_m_pi += (1 / (tau + 1)) * (jnp.log(mass_x) + jnp.log(mass_y) + ldet_c
+                                   + 0.5 * (tau * ldet_t_ab - ldet_ab))
+
+    log_m_pi += -jnp.sum(diff_means * jnp.linalg.solve(
+        x_mat + y_mat + lam * iden, diff_means)) / (2 * (tau + 1))
+
+    log_m_pi += - 0.5 * ldet_c_ab
+
+    # If all logdet signs are 1, output value, nan otherwise.
+    return jnp.where(
+        sldet_c == 1 and sldet_c_ab == 1 and sldet_ab == 1 and sldet_t_ab == 1,
+        2 * sig2 * mass_x * mass_y - 2 * (sig2 + gam) * jnp.exp(log_m_pi),
+        jnp.nan)
+
+  def tree_flatten(self):
+    return (), (self._dimension, self._gamma, self._sigma2, self._sqrtm_kw)
+
+  @classmethod
+  def tree_unflatten(cls, aux_data, children):
+    del children
+    return cls(aux_data[0], aux_data[1], aux_data[2], **aux_data[3])
